package no.ntnu.idi.bacchus;

import java.awt.Dimension;
import java.io.*;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.*;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;

import org.apache.derby.tools.ij;
import org.hibernate.SessionFactory;
import org.hibernate.cfg.Configuration;
import org.hibernate.classic.Session;
import org.hibernate.tool.hbm2ddl.SchemaExport;

import es.ucm.fdi.gaia.ontobridge.OntoBridge;
import es.ucm.fdi.gaia.ontobridge.OntologyDocument;

import no.ntnu.idi.bacchus.db.DataBaseConnector;
import no.ntnu.idi.bacchus.domain.MealDescription;
import no.ntnu.idi.bacchus.explanation.MealExplanation;
import no.ntnu.idi.bacchus.gui.*;
import no.ntnu.idi.bacchus.bn.BNScoringMethod;
import no.ntnu.idi.bacchus.bn.MealsNetwork;
import no.ntnu.idi.bacchus.bn.ProbabilityScoring;
import no.ntnu.idi.bacchus.bn.SimilarityConfig;
import no.ntnu.idi.bacchus.config.MealCaseBase;

import jcolibri.casebase.LinealCaseBase;
import jcolibri.cbrcore.Attribute;
import jcolibri.cbrcore.CBRCase;
import jcolibri.cbrcore.CBRCaseBase;
import jcolibri.cbrcore.CBRQuery;
import jcolibri.cbrcore.Connector;
//import jcolibri.connector.DataBaseConnector;
import jcolibri.exception.ExecutionException;
import jcolibri.method.retrieve.RetrievalResult;
import jcolibri.method.retrieve.NNretrieval.NNConfig;
import jcolibri.method.retrieve.NNretrieval.NNScoringMethod;
import jcolibri.method.retrieve.NNretrieval.similarity.global.Average;
import jcolibri.method.retrieve.selection.SelectCases;
import jcolibri.util.FileIO;

public class MealRecommender  implements StandardCBRApplication{
	
	private static MealRecommender instance = null;
	MealManager mealManager;
	CBRCaseBase casebase;
	Connector connector;
	Connection conn;
	
	SimilarityDialog similarityDialog;
	ResultDialog resultDialog;
	AutoAdaptationDialog autoAdaptDialog;
	RevisionDialog revisionDialog;
	RetainDialog retainDialog;
	ExplanationDialog explanationDialog;
	
	String explanation;
	public MealsNetwork mealsNetwork;
	public static ProbabilityScoring probScoring;
	
	
	public static MealRecommender getInstance(){
		if(instance == null)
			instance = new MealRecommender();
		return instance;
	}
	
	protected boolean runScript(File scriptFile, Connection connection) {
		   FileInputStream fileStream = null;
		   try {
		       fileStream = new FileInputStream(scriptFile);
		       int result = ij.runScript(connection,fileStream,"UTF-8",System.out,"UTF-8");

		       System.out.println("Result code is: " + result);
		       return (result==0);
		   }
		   catch (FileNotFoundException e) {
			   System.out.println("File not found");
		       return false;
		   }
		   catch (UnsupportedEncodingException e) {
			   System.out.println("Unsupported file");
		       return false;
		   }
		   finally {
		       if(fileStream!=null) {
		           try {
		               fileStream.close();
		           }
		           catch (IOException e) {
		           }
		       }
		   }
		}

	
	@Override
	public void configure() throws ExecutionException {
		try{
			System.out.println("Configuring...");
			
			Class.forName("org.apache.derby.jdbc.EmbeddedDriver");
			File file = new File(FileIO.findFile("no/ntnu/idi/bacchus/db/meal.sql").getFile());
		    conn = DriverManager.getConnection("jdbc:derby:bacchus;create=true", "bacchus", "ivv");
		    System.out.println("\nRunning SQL-script:");
		    runScript(file, conn);
			
			connector = new DataBaseConnector();
			System.out.println("\nRunning config-files");
			connector.initFromXMLfile(jcolibri.util.FileIO.findFile("no/ntnu/idi/bacchus/config/databaseconfig.xml"));
			casebase = new LinealCaseBase();
			
			//Obtain a reference to OntoBridge
			OntoBridge ob = jcolibri.util.OntoBridgeSingleton.getOntoBridge();
			//Configure it to work with the Pellet reasoner
			ob.initWithPelletReasoner();
			//Setup the main ontology
			OntologyDocument foodOntology = new OntologyDocument("http://www.owl-ontologies.com/Ontology1268125572_food.owl", FileIO.findFile("resources/ontology_food.owl").toExternalForm());		
			//There are not subontologies (?)
			OntologyDocument accOntology = new OntologyDocument("http://www.owl-ontologies.com/Ontology1268125572_acc.owl", FileIO.findFile("resources/ontology_accessories.owl").toExternalForm());
			ArrayList<OntologyDocument> subOntologies = new ArrayList <OntologyDocument>();
			subOntologies.add(accOntology);
			//Load the ontology
			ob.loadOntology(foodOntology, subOntologies, true);
			
			
			// Create the dialogs
			similarityDialog = new SimilarityDialog(main);
			resultDialog     = new ResultDialog(main);
			autoAdaptDialog  = new AutoAdaptationDialog(main);
			revisionDialog   = new RevisionDialog(main);
			retainDialog     = new RetainDialog(main);
			explanationDialog = new ExplanationDialog(main);
		}
		catch(Exception e){
			throw new ExecutionException(e);
		}
	}
	
	@Override
	public CBRCaseBase preCycle() throws ExecutionException {
		
		System.out.println("\nPrinting out casebase: ");
		casebase.init(connector);
		for(jcolibri.cbrcore.CBRCase c: casebase.getCases())
			System.out.println(c);
		return casebase;
	}

	public static ProbabilityScoring getProbScoring() {
		return probScoring;
	}

	@Override
	public void cycle(CBRQuery query) throws ExecutionException {
		
		//Initiale BN
		mealsNetwork = new MealsNetwork();
		//The first BN was named redwine_food_workingcopy, the first revision gave us redwine_food_extended_workingcopy and the last revision is newGrapeValues.
        String redWineFile = mealsNetwork.getFullPath("redwine_food_extended_workingcopy_newGrapeValues.xdsl");
        mealsNetwork.initNetwork(redWineFile);
        mealsNetwork.setUpdateValue(0.10);
		
		System.out.println("Query: " + query +"\n");
		
		// Obtain configuration for similarity
		similarityDialog.setVisible(true);
		
		NNConfig simConfig = similarityDialog.getSimilarityConfig();
		simConfig.setDescriptionSimFunction(new Average()); 
		
		//Get the BN-cases
		probScoring = new ProbabilityScoring(mealsNetwork);
		Collection<CBRCase> bnCases = new ArrayList<CBRCase>();
		bnCases = probScoring.runProbabilityScoring(casebase.getCases(), query, similarityDialog.getI());
		
		//Execute similarity
		Collection<RetrievalResult> eval = NNScoringMethod.evaluateSimilarity(bnCases, query, simConfig);
		
		//Select k cases
		Collection<CBRCase> selectedCases = SelectCases.selectTopK(eval, similarityDialog.getK());

		//Show results!
		resultDialog.showCases(eval, selectedCases, query);
		resultDialog.setVisible(true);

	}
	


	@Override
	public void postCycle() throws ExecutionException {
		connector.close();
		//shutdown();
	}
	
    private void shutdown(){
        try{
            if (conn != null){
                DriverManager.getConnection("jdbc:derby:bacchus;shutdown=true");
                conn.close();
            }           
        }
        catch (SQLException sqlExcept)
        {}
    }


	static JFrame main;
	void showMainFrame()
	{
		main = new JFrame("Meal Recommender");
		main.setUndecorated(true);
		JLabel label = new JLabel(new ImageIcon(jcolibri.util.FileIO.findFile("resources/wine_food.jpg")));
		main.getContentPane().add(label);
		main.pack();
		Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
		main.setBounds((screenSize.width - main.getWidth()) / 2,
				(screenSize.height - main.getHeight()) / 2, 
				main.getWidth(),
				main.getHeight());
		main.setVisible(true);
	}
	
	public static void main(String[] args){
		//Obtain WineRecommender object
		MealRecommender recommender = getInstance();
		recommender.showMainFrame();
		try{
			//Configure the application
			recommender.configure();
			
			//Execute the precycle
			recommender.preCycle();
			
			QueryDialog qf = new QueryDialog(main);
			boolean cont = true;
			while(cont)
			{
				qf.setVisible(true);
				CBRQuery query = qf.getQuery();
			
				//Call the cycle
				recommender.cycle(query);
				int ans = javax.swing.JOptionPane.showConfirmDialog(null, "CBR cycle finished, query again?", "Cycle finished", javax.swing.JOptionPane.YES_NO_OPTION);
				if(ans ==javax.swing.JOptionPane.YES_OPTION)
					cont = true;
				else if(ans == javax.swing.JOptionPane.NO_OPTION)
					cont = false;
			}
			recommender.postCycle();
		}
		catch(Exception e){
			//Errors
			System.out.println("Error: " + e.toString());
		}
		System.exit(0);
	}
}