package no.ntnu.idi.bacchus.bn;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import jcolibri.cbrcore.CBRCase;
import jcolibri.cbrcore.CBRQuery;
import jcolibri.cbrcore.CaseComponent;
import jcolibri.method.retrieve.RetrievalResult;
import jcolibri.method.retrieve.NNretrieval.NNConfig;
import jcolibri.method.retrieve.NNretrieval.NNScoringMethod;
import jcolibri.method.retrieve.NNretrieval.similarity.GlobalSimilarityFunction;
import jcolibri.util.ProgressController;

public class BNScoringMethod {

	@SuppressWarnings("unchecked")
    /**
     * Performs the BN scoring over a collection of cases comparing them with a query. 
     * This method is configured through the SimilarityConfig object.
     */
	public static Collection<RetrievalResult> evaluateSimilarity(Collection<CBRCase> cases, CBRQuery query, NNConfig simConfig){
		//TODO: FIX!
		List<RetrievalResult> res = new ArrayList<RetrievalResult>();
		ProgressController.init(BNScoringMethod.class,"Numeric Similarity Computation", cases.size());
		GlobalSimilarityFunction gsf = simConfig.getDescriptionSimFunction(); // set to ProbabilityScoring in MealRecommender
		HashMap<String,Double> kBNValues = ((ProbabilityScoring) gsf).getKHighestBNValues(query, 3);
		Collection<CBRCase> bnCases = new ArrayList<CBRCase>();
		//Put the relevant cases in the bnCases arraylist
		bnCases = ((ProbabilityScoring) gsf).getRelevantCases(cases, kBNValues);
		//Calculates the similarity for each case
		for(CBRCase _case: bnCases){
			res.add(new RetrievalResult(_case, gsf.compute(_case.getDescription(), query.getDescription(), _case, query, simConfig)));
			//res.add(new RetrievalResult(_case, Math.random()));
			ProgressController.step(NNScoringMethod.class);
		}
		java.util.Collections.sort(res);
		ProgressController.finish(NNScoringMethod.class);
		
		return res;
	}
	
	 /**
     * Selects top K cases
     * @param cases to select
     * @param k is the number of csaes to select
     * @return top k cases
     */
    public static Collection<CBRCase> selectTopK(Collection<RetrievalResult> cases, int k){
		//TODO: FIX!!!
    	ArrayList<CBRCase> res = new ArrayList<CBRCase>();
		Iterator<RetrievalResult> cIter  =cases.iterator(); 
		for(int c=0; c<k && c<cases.size(); c++)
		    res.add(cIter.next().get_case());
		return res;    
    }
	
	
}
