package no.ntnu.idi.bacchus.bn;

import java.io.File;
import java.io.IOException;

import jcolibri.datatypes.Instance;

import smile.DiagNetwork;
import smile.DiagResults;
import smile.FaultInfo;
import smile.Network;
import smile.ObservationInfo;
import smile.SMILEException;

public class MealNetwork {

private static String path;	
private Network redWineNetwork;
private Instance meatType;
private double updateValue;

	/**
	 * This method sets the updateValue - i.e. the value a node should be updated with
	 * @param updateValue 
	 */
	public void setUpdateValue(double updateValue) {
		this.updateValue = updateValue;
	}

	/**
	 * This method returns the updateValue
	 * @return updateValue
	 */
	public double getUpdateValue() {
		return updateValue;
	}

	/**
	 * The method returns the full path of the file
	 * @param coreName
	 * @return the path of the BN file
	 */
	public String getFullPath(String coreName){
		//System.out.println("BN-File Path: " +path+coreName);
		File file = new File(""); 
        String path = file.getAbsolutePath() + "\\resources\\";
		return path + coreName;
	}
	
	/**
	 * This method initiates the network.
	 * @param networkName
	 */
	public void initNetwork(String networkName){
		redWineNetwork = new Network();
		redWineNetwork.readFile(networkName);
		redWineNetwork.updateBeliefs();	
	}
	
	/**
	 * This method returns the index of the given nodeValue of the outcome 
	 * @param outcomeId - the id of the outcome (ex: MeatType)
	 * @param nodeValue - the value of the node (ex: LightMeat)
	 * @return the outcomeIndex  (ex: index of MeatTypes LightMeat is 0)
	 */
	public int getOutcomeIndex(String outcomeId, String nodeValue){
		//Getting the index of the outcome
		String[] outcommeIds = redWineNetwork.getOutcomeIds(outcomeId);
		int outcomeIndex;
		for(outcomeIndex = 0; outcomeIndex < outcommeIds.length; outcomeIndex++)
			if(nodeValue.equals(outcommeIds[outcomeIndex]))
				return outcomeIndex;
		return 99; //FIX!
		
	}
	/**
	 * This method calculates the probabilities of the given nodes
	 * @param outcomeId - The ID of the node, ex: Richness
	 * @param nodeValue - The value of the node, ex: Rich
	 * @param evidenceNodeId - The evidence node id, ex: MeatType
	 * @param evidenceNodeValue - The evidence node value, ex: Beef
	 * @param clearEvidence - If evidence should be cleared
	 * @return the probability of the node
	 */
	public double getNodeProb(String outcomeId, String nodeValue, String evidenceNodeId, String evidenceNodeValue, boolean clearEvidence){
		if(evidenceNodeId!=null && evidenceNodeValue!=null){
			//Set the evidence values
			if(clearEvidence)
				redWineNetwork.clearEvidence(evidenceNodeId);
			redWineNetwork.setEvidence(evidenceNodeId, evidenceNodeValue);
			redWineNetwork.updateBeliefs();
		}
		//Getting the value of the propability
		double[] values = redWineNetwork.getNodeValue(outcomeId); //Values[] keeps all the values 
		double probability = values[getOutcomeIndex(outcomeId, nodeValue)];
		return probability;
	}
	
	/**
	 * This method print the probability of the given node
	 * @param outcomeId - The ID of the node, ex: Richness
	 * @param nodeValue - The value of the node, ex: Rich
	 * @param evidenceNodeId - The evidence node id, ex: MeatType
	 * @param evidenceNodeValue - The evidence node value, ex: Beef
	 */
	public void printProbability(String outcomeId, String nodeValue, String evidenceNodeId, String evidenceNodeValue){
		double probability = getNodeProb(outcomeId, nodeValue, evidenceNodeId, evidenceNodeValue, false);
		if(evidenceNodeId!=null && evidenceNodeValue!=null)
			System.out.println("P("+outcomeId+"="+nodeValue+" | "+evidenceNodeId+"="+evidenceNodeValue+")= " + probability);
		else
			System.out.println("P("+outcomeId+"="+nodeValue+")= " + probability);
	}
	
	/**
	 * This method updates the network 
	 * @param outcomeId
	 * @param nodeValue
	 */
	public void updateNetwork(String outcomeId, String nodeValue){
		//Getting the array with the probabilities
		double[] values = redWineNetwork.getNodeValue(outcomeId); //Values[] keeps all the values 
		//Update the array
		double[] updatedValues = new double[values.length];
		double updateWith = updateValue / (values.length-1);
		int outcomeIndex = getOutcomeIndex(outcomeId, nodeValue);
		System.out.println("\nPrinting values: ");
		for(int i = 0; i < values.length; i++){
			if(i == outcomeIndex)
				updatedValues[i] = values[i]+ updateValue;
			else
				updatedValues[i] = values[i] - updateWith;
			//System.out.println("Value "+i+": "+values[i]+" updated to: " + updatedValues[i]);
		}
		redWineNetwork.setNodeDefinition(outcomeId, updatedValues);
		redWineNetwork.updateBeliefs();
	}


	public static void main(String[] args) throws IOException{
    
		MealNetwork mealNetwork = new MealNetwork();
        //File used to get fullpath
        File file = new File(""); 
        path = file.getAbsolutePath() + "\\resources\\";
        
        String redWineFile = mealNetwork.getFullPath("redwine_food_test.xdsl");
        mealNetwork.initNetwork(redWineFile);
        mealNetwork.setUpdateValue(0.10);
        
    	System.out.println("Printing out Meat values:");
    	
		mealNetwork.printProbability("MeatType", "LightMeat", null, null);
		mealNetwork.updateNetwork("MeatType", "LightMeat");
		mealNetwork.printProbability("MeatType", "LightMeat", null, null);
		mealNetwork.printProbability("MeatType", "Beef", null, null);
		
		/**getNodeProb("MeatType", "Beef", null, null, false);
		getNodeProb("MeatType", "Pork", null, null, false);
		getNodeProb("MeatType", "Lamb_Sheep", null, null, false);
		getNodeProb("MeatType", "Fish", null, null, false);
		getNodeProb("MeatType", "Cheese", null, null, false);
		getNodeProb("MeatType", "Small_game_Bird", null, null, false);
		getNodeProb("MeatType", "Big_game", null, null, false);
		getNodeProb("MeatType", "Vegetables", null, null, false);
		
		getNodeProb("Richness", "Rich", "MeatType", "Beef", false);
		getNodeProb("GrapeType", "PinotNoir", "MeatType", "Beef", false);
		getNodeProb("GrapeType", "Barbera", "MeatType", "Beef", false);**/
	}
}
