package no.ntnu.idi.bacchus.bn;

import java.io.File;
import java.io.IOException;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

import jcolibri.datatypes.Instance;

import smile.DiagNetwork;
import smile.DiagResults;
import smile.FaultInfo;
import smile.Network;
import smile.ObservationInfo;
import smile.SMILEException;

public class MealsNetwork extends Network{

	private static String path;
	private String networkname;
	private Instance foodType;
	private double updateValue;
	private int foodTypes;


	private String getNetworkname() {
		return networkname;
	}


	private void setNetworkname(String networkname) {
		this.networkname = networkname;
	}


	/**
	 * This method sets the updateValue - i.e. the value a node should be updated with
	 * @param updateValue
	 */
	public void setUpdateValue(double updateValue) {
		this.updateValue = updateValue;
	}
	

	/**
	 * This method returns the updateValue
	 * @return updateValue
	 */
	public double getUpdateValue() {
		return updateValue;
	}
	
	public int getFoodTypes() {
		return foodTypes;
	}


	public void setFoodTypes(int foodTypes) {
		this.foodTypes = foodTypes;
	}


	/**
	 * The method returns the full path of the file
	 * @param coreName
	 * @return the path of the BN file
	 */
	public String getFullPath(String coreName){
		//System.out.println("BN-File Path: " +path+coreName);
		File file = new File("");
        String path = file.getAbsolutePath() + "\\resources\\";
		return path + coreName;
	}

	/**
	 * This method initiates the network.
	 * @param networkName
	 */
	public void initNetwork(String networkName){
		
		System.out.println("Initiating BNnetwork..");
		setNetworkname(networkName);
		readFile(networkName);
		updateBeliefs();
		
		this.foodTypes = getOutcomeCount("FoodType");
	}
	
	
	
	/**
	 * This method calculates the probabilities of the given nodes
	 * @param outcomeId - The ID of the node, ex: Richness
	 * @param nodeValue - The value of the node, ex: Rich
	 * @param evidenceNodeId - The evidence node id, ex: MeatType
	 * @param evidenceNodeValue - The evidence node value, ex: Beef
	 * @param clearEvidence - If evidence should be cleared
	 * @return the probability of the node
	 */
	public double getNodeProb(String outcomeId, String nodeValue, String evidenceNodeId, String evidenceNodeValue, boolean clearEvidence){
		if(clearEvidence){
			clearAllEvidence();
			updateBeliefs();
		}
		if(evidenceNodeId!=null && evidenceNodeValue!=null){
			//Set the evidence values
			setEvidence(evidenceNodeId, evidenceNodeValue);
			updateBeliefs();
		}
		//Getting the value of the probability
		double[] values = getNodeValue(outcomeId); //Values[] keeps all the values
		double probability = values[getOutcomeIndex(outcomeId, nodeValue)];
		return probability;
	}
	
	public HashMap<String,Double> getNodeProbForAllGrapes(String[] outcomeIds, String nodeValue){
		Map<String,Double> grapeProbabilities = new HashMap<String,Double>();
		for(int i = 0; i < outcomeIds.length; i++){
			double[] values = getNodeValue(outcomeIds[i]); //Values[] keeps all the values
			double probability = values[getOutcomeIndex(outcomeIds[i], nodeValue)];
			grapeProbabilities.put(outcomeIds[i], probability); //Key: Grape, Value: probability
		}
		return (HashMap<String, Double>) grapeProbabilities;
	}

	public HashMap<String,Double> getAllNodeProbabilites(String outcomeId, String[] nodeValues) {
		Map<String,Double> nodeProbabilities = new HashMap<String,Double>();
		double[] values = getNodeValue(outcomeId);
		for(int j = 1; j < nodeValues.length; j++) {
			double probability = values[getOutcomeIndex(outcomeId, nodeValues[j])];
			nodeProbabilities.put(outcomeId + ": " + nodeValues[j], probability);
			System.out.println("NodeValue "+ nodeValues[j] + " returned probability "+ probability);
			}			
		return (HashMap<String, Double>) nodeProbabilities;
	}
	
	
	public int getSpecificOutcomeIndex(String outcomeId, String nodeValue){
		//Getting the index of the outcome
		String[] outcommeIds = getOutcomeIds(outcomeId);
		int outcomeIndex;
		for(outcomeIndex = 0; outcomeIndex < outcommeIds.length; outcomeIndex++) {
			if(outcomeId.equals("Bitterness") && nodeValue.equals("High") || outcomeId.equals("Bitterness") && nodeValue.equals("Medium") || 
					outcomeId.equals("Bitterness") &&  nodeValue.equals("Low"))
				return outcomeIndex;
			else if(outcomeId.equals("Freshness") && nodeValue.equals("High") || outcomeId.equals("Freshness") && nodeValue.equals("Medium") || 
					outcomeId.equals("Freshness") &&  nodeValue.equals("Low"))
				return outcomeIndex;
			else if(outcomeId.equals("Flavor") && nodeValue.equals("Strong") || outcomeId.equals("Flavor") && nodeValue.equals("Medium") || 
					outcomeId.equals("Flavor") &&  nodeValue.equals("Bland"))
				return outcomeIndex;
			else if(outcomeId.equals("Richness") && nodeValue.equals("Rich") || outcomeId.equals("Richness") && nodeValue.equals("Good") || 
					outcomeId.equals("Richness") &&  nodeValue.equals("Medium") || outcomeId.equals("Richness") &&  nodeValue.equals("Light"))
				return outcomeIndex;
			else if(outcomeId.equals("Fruitiness") && nodeValue.equals("Rich") || outcomeId.equals("Fruitiness") && nodeValue.equals("Good") || 
					outcomeId.equals("Fruitiness") &&  nodeValue.equals("Medium") || outcomeId.equals("Fruitiness") &&  nodeValue.equals("Light"))
				return outcomeIndex;
			else if(outcomeId.equals("TanningAgents") && nodeValue.equals("Good") || outcomeId.equals("TanningAgents") && nodeValue.equals("Medium") || 
					outcomeId.equals("TanningAgents") &&  nodeValue.equals("Some"))
				return outcomeIndex;	
			else
				break; 
		}
		return -1;
	}
	
	/**
	 * This method print the probability of the given node
	 * @param outcomeId - The ID of the node, ex: Richness
	 * @param nodeValue - The value of the node, ex: Rich
	 * @param evidenceNodeId - The evidence node id, ex: MeatType
	 * @param evidenceNodeValue - The evidence node value, ex: Beef
	 */
	public void printProbability(String outcomeId, String nodeValue, String evidenceNodeId, String evidenceNodeValue){
		double probability = getNodeProb(outcomeId, nodeValue, evidenceNodeId, evidenceNodeValue, false);
		if(evidenceNodeId!=null && evidenceNodeValue!=null)
			System.out.println("P("+outcomeId+"="+nodeValue+" | "+evidenceNodeId+"="+evidenceNodeValue+")= " + probability);
		else
			System.out.println("P("+outcomeId+"="+nodeValue+")= " + probability);
	}

	/**
	 * This method updates the network
	 * @param outcomeId
	 * @param nodeValue
	 */
	public void updateNodeProb(String outcomeId, String nodeValue){
		//Getting the array with the probabilities
		double[] values = getNodeValue(outcomeId); //Values[] keeps all the values
		//Update the array
		double[] updatedValues = new double[values.length];
		double updateWith = updateValue / (values.length-1);
		int outcomeIndex = getOutcomeIndex(outcomeId, nodeValue);
		System.out.println("\nPrinting values: ");
		for(int i = 0; i < values.length; i++){
			if(i == outcomeIndex)
				updatedValues[i] = values[i]+ updateValue;
			else
				updatedValues[i] = values[i] - updateWith;
			//System.out.println("Value "+i+": "+values[i]+" updated to: " + updatedValues[i]);
		}
		setNodeDefinition(outcomeId, updatedValues);
		updateBeliefs();
	}
	
	/**
	 * This method returns the index of the given nodeValue of the outcome
	 * @param outcomeId - the id of the outcome (ex: FoodType)
	 * @param nodeValue - the value of the node (ex: LigttMeat)
	 * @return the outcomeIndex  (ex: index of FoodTypes LightMeat is 0)
	 */
	public int getOutcomeIndex(String outcomeId, String nodeValue){
		//Getting the index of the outcome
		String[] outcommeIds = getOutcomeIds(outcomeId);
		int outcomeIndex;
		for(outcomeIndex = 0; outcomeIndex < outcommeIds.length; outcomeIndex++)
			if(nodeValue.equals(outcommeIds[outcomeIndex]))
				return outcomeIndex;
		return -1; 
	}
	
	public int getHighestNodeState(String node){
		double[] values = getNodeValue(node);
		int highestState = 0;
		double highestValue = 0.0;
		for(int i = 0; i < values.length; i++){
			if(values[i] > highestValue){
				highestValue = values[i];
				highestState = i;
			}
		}
		return highestState;
		
	}
	
	/**
	 * This method finds the relevant index (where the updating will be done) of the probability table
	 * @param nodeId - The node to be updated
	 * @param foodType - The given foodtype
	 * @param accType - The given accessoriesType
	 * @param sauceType - The given sauceType
	 * @return the index where the updating should start (the first value in a column)
	 */
	public int getUpdateNodeIndex(String nodeId, String foodType, int bitterness, int freshness, int flavor){
		int nodeIndex = 0;

		System.out.println("Node bitterness: " + bitterness + " freshness: " + freshness +" flavor: " + flavor);
		//Finds the relevant outcomeIndexes
		int foodTableIndex = getOutcomeIndex("FoodType", foodType);
		int bitternessIndex = bitterness;
		int freshnessIndex = freshness;
		int flavorIndex = flavor;
		
		int bitternessOutcomes = getOutcomeCount("Bitterness");
		int freshnessOutcomes = getOutcomeCount("Freshness");
		int flavorOutcomes = getOutcomeCount("Flavor");
		
		//The number of outcomes of the node
		int nodeOutcomes = getOutcomeCount(nodeId);
		
		if(nodeId.equals("Richness")){
			//Parental nodes: Food, Flavor
			nodeIndex = (foodTableIndex * flavorOutcomes * nodeOutcomes) + (flavorIndex * nodeOutcomes);
		}
		else if(nodeId.equals("TanningAgents")){
			//Parental nodes: Food, Bitterness, Flavor
			nodeIndex = (foodTableIndex * bitternessOutcomes * flavorOutcomes * nodeOutcomes) + (bitternessIndex * flavorOutcomes *nodeOutcomes) + (flavorIndex * nodeOutcomes);
		}
		else if(nodeId.equals("Fruitiness")){
			//Parental nodes: Food, Freshness
			nodeIndex = (foodTableIndex * freshnessOutcomes * nodeOutcomes) + (freshnessIndex * nodeOutcomes);
		}

		System.out.println("Nodeindex for " + nodeId + "("+foodType+") :" + nodeIndex);
		
		double[] values = getNodeDefinition(nodeId);
		System.out.println("Total values: " + values.length);
		System.out.println("Node prob: " + values[nodeIndex]);
		
		return nodeIndex;
	}

	
	//TODO: Currently not working!
	public void updateNetwork(String outcomeId, String nodeValue, String foodType, String accType, String sauceType){
		System.out.println("\nUpdating outcome "+outcomeId+":"+nodeValue+" given "+foodType+", "+accType+" and "+sauceType);
		
		//Set evidence first.
		setEvidence("FoodType", foodType);
		setEvidence("AccessoriesType", accType);
		setEvidence("SauceType", sauceType);
		updateBeliefs();
		
		//Hent inn verdiene p bitterness, freshness og flavor
		
		
		//The number of outcomes of the node
		int outcomes = getOutcomeCount(outcomeId);
		
		
		
		int nodeIndex = getUpdateNodeIndex(outcomeId, foodType, getHighestNodeState("Bitterness"), getHighestNodeState("Freshness"), getHighestNodeState("Flavor"));
		double[] values = getNodeDefinition(outcomeId);
		
		//the index that should be updated
		int outcomeIndex = getOutcomeIndex(outcomeId, nodeValue) + nodeIndex;
		double updateWith = updateValue / (outcomes-1);
		
		System.out.println("Update with: " + updateWith);
		
		//Check if values will be below zero
		for(int j = nodeIndex; j < nodeIndex+outcomes; j++){
			if(j == outcomeIndex && (values[j] + updateValue) > 1.0)
				setUpdateValue(updateWith);
			if(j != outcomeIndex && (values[j] - updateWith) < 0.0){
				double diff = values[j] - updateWith;
				updateWith += Math.abs(diff)/2;
				System.out.println("Neg value => Update with is now: " + updateWith+ " diff: " + diff);
			}	
		}
		
		System.out.println("Index: " + nodeIndex +" of total " + values.length);
		//System.out.println("Node prob: " + values[nodeIndex]);
		
		//The array holding the new values 
		double[] updatedValues = Arrays.copyOf(values, values.length);
		
		/**for(int i = nodeIndex; i < nodeIndex+outcomes; i++){
			if(i == outcomeIndex)
				updatedValues[i] = values[i]+ updateValue;
			else {
				if((values[i] - updateWith) < 0)
					updatedValues[i] = 0.0;
				else
					updatedValues[i] = values[i] - updateWith;
			}
			System.out.println("Value "+i+": "+values[i]+" updated to: " + updatedValues[i]);
		}
		setNodeDefinition(outcomeId, updatedValues);
		updateBeliefs();
		writeFile(getFullPath("test.xdsl"));**/
	}


	public static void main(String[] args) throws IOException{

		MealsNetwork mealsNetwork = new MealsNetwork();
        //File used to get fullpath
        File file = new File("");
        path = file.getAbsolutePath() + "\\resources\\";

        String redWineFile = mealsNetwork.getFullPath("redwine_food_extended_test.xdsl");
        //String redWineFile = mealsNetwork.getFullPath("test.xdsl");
        
        mealsNetwork.initNetwork(redWineFile);
        mealsNetwork.setUpdateValue(0.05);
        

		

	}
}
