package no.ntnu.idi.bacchus.bn;

import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Set;

import no.ntnu.idi.bacchus.domain.GrapeDescription;
import no.ntnu.idi.bacchus.domain.MealSolution;
import no.ntnu.idi.bacchus.explanation.MealExplanation;

import smile.Network;

import es.ucm.fdi.gaia.ontobridge.OntoBridge;
import jcolibri.cbrcore.Attribute;
import jcolibri.cbrcore.CBRCase;
import jcolibri.cbrcore.CBRQuery;
import jcolibri.cbrcore.CaseComponent;
import jcolibri.datatypes.Instance;
import jcolibri.method.retrieve.RetrievalResult;
import jcolibri.method.retrieve.NNretrieval.NNConfig;
import jcolibri.method.retrieve.NNretrieval.NNScoringMethod;
import jcolibri.method.retrieve.NNretrieval.similarity.GlobalSimilarityFunction;
import jcolibri.method.retrieve.NNretrieval.similarity.InContextLocalSimilarityFunction;
import jcolibri.method.retrieve.NNretrieval.similarity.LocalSimilarityFunction;
import jcolibri.util.ProgressController;

public class ProbabilityScoring{

//	public enum ArrayTypes { Bitterness, Freshness, Flavor, Richness, TanningAgents, Fruitiness }
	
//	ArrayTypes arrayType;
	
	int arrayTypeInt;
	
	private MealsNetwork mealsNetwork;
	private OntoBridge ob;
	private String foodOntoClass;
	private String accOntoClass;
	private String sauceClassName;
	private String[] wineGrapes;
	private String[] richnessArray;
	private String[] fruitinessArray;
	private String[] bitternessArray;
	private String[] freshnessArray;
	private String[] flavorArray;
	private String[] tanningAgentsArray;
	String[] shortArray;
	String[] longArray;
	
	
	public ProbabilityScoring(MealsNetwork mealsNetwork){
		this.mealsNetwork = mealsNetwork;
		//Get a reference to OntoBridge
		ob = jcolibri.util.OntoBridgeSingleton.getOntoBridge();
		foodOntoClass = "FoodType"; //Must be the same as the name in the Ontology!!
		accOntoClass = "AccessoriesType"; //Must be the same as the name in the Ontology!!
		sauceClassName = "SauceType";
		initWineGrapeArray();
		initConditionsArrays();
//		initNodeValuesArray();
		shortArray = new String[4];
		longArray = new String[5];
		
		
	}
	
	public void initWineGrapeArray(){
		wineGrapes = new String[16];
		wineGrapes[0] = "Barbera";
		wineGrapes[1] = "CabernetFranc";
		wineGrapes[2] = "CabernetSauvignon";
		wineGrapes[3] = "Carignan";
		wineGrapes[4] = "Corvina";
		wineGrapes[5] = "Grenache";
		wineGrapes[6] = "Malbec";
		wineGrapes[7] = "Merlot";
		wineGrapes[8] = "Montepulciano";
		wineGrapes[9] = "PinotNoir";
		wineGrapes[10] = "Rondinella";
		wineGrapes[11] = "Sangiovese";
		wineGrapes[12] = "Syrah";
		wineGrapes[13] = "Tempranillo";
		wineGrapes[14] = "TintaMiuda";
		wineGrapes[15] = "Zinfandel";
	}
	
	public void initConditionsArrays() {
		bitternessArray = new String[4];
		bitternessArray[0] = "Bitterness";
		bitternessArray[1] = "Low";
		bitternessArray[2] = "Medium";
		bitternessArray[3] = "High";
		
		freshnessArray = new String[4];
		freshnessArray[0] = "Freshness";
		freshnessArray[1] = "Low";
		freshnessArray[2] = "Medium";
		freshnessArray[3] = "High";
		
		flavorArray = new String[4];
		flavorArray[0] = "Flavor";
		flavorArray[1] = "Bland";
		flavorArray[2] = "Medium";
		flavorArray[3] = "Strong";
		
		richnessArray = new String[5];
		richnessArray[0] = "Richness";
		richnessArray[1] = "Light";
		richnessArray[2] = "Medium";
		richnessArray[3] = "Good";
		richnessArray[4] = "Rich";
		
		fruitinessArray = new String[5];
		fruitinessArray[0] = "Fruitiness";
		fruitinessArray[1] = "Light";
		fruitinessArray[2] = "Medium";
		fruitinessArray[3] = "Good";
		fruitinessArray[4] = "Rich";
		
		tanningAgentsArray = new String[4];
		tanningAgentsArray[0] = "TanningAgents";
		tanningAgentsArray[1] = "Some";
		tanningAgentsArray[2] = "Medium";
		tanningAgentsArray[3] = "Good";
	}
	
/*
	public void initNodeValuesArray() {
		nodeNameArray = new String[6];
		nodeNameArray[0] = "Flavor";
		nodeNameArray[1] = "Richness";
		nodeNameArray[2] = "Bitterness";
		nodeNameArray[3] = "TanningAgents";
		nodeNameArray[4] = "Freshness";
		nodeNameArray[5] = "Fruitiness";
	}
*/
	
	public String[] getArray(int type) {
		arrayTypeInt = type;
		
		if(type <= 5) 
			switch(arrayTypeInt) {
				case 0: shortArray = bitternessArray; return shortArray;
				case 1: shortArray = freshnessArray; return shortArray;
				case 2: shortArray = flavorArray; return shortArray;
				case 3: longArray = richnessArray; return longArray;
				case 4: shortArray = tanningAgentsArray; return shortArray;
				case 5: longArray = fruitinessArray; return longArray;
			}

		return shortArray;
	}
	
	public String getOntoClass(String ontoValue){
		Iterator<String> ontoNames = ob.listBelongingClasses(ontoValue);
		while(ontoNames.hasNext()){
			String ontoName = ontoNames.next();
			String className = ontoName.substring(ontoName.indexOf("#")+1);
			if(!(className.equals(foodOntoClass) || className.equals(accOntoClass) || className.equals("Thing") || className.equals("Class"))){
				System.out.println("OntoClas: " + className);
				if(ob.isSubClassOf(className, "LightMeat")){
					System.out.println("Onto: " + className + " is instance of LightMeat");
					return "LightMeat";
				}
				return className;
			}	
		}
		return ontoValue;
	}
	
	public String[] getAccOntoClasses(String ontoValue){
		String temp = ontoValue.substring(ontoValue.indexOf("[")+1, ontoValue.lastIndexOf("]"));
		String[] accValues = temp.split(", ");
		String[] ontoClasses = new String[accValues.length];
		for(int i = 0; i < accValues.length; i++){
			ontoClasses[i] = getOntoClass(accValues[i]);
		}
		return ontoClasses;
	}
	
	
	public MealsNetwork getMealNetwork() {
		return mealsNetwork;
	}
	
	public void setMealNetwork(MealsNetwork mealsNetwork) {
		this.mealsNetwork = mealsNetwork;
	}
	
	@SuppressWarnings("unchecked")
    /**
     * Performs the BN scoring over a collection of cases comparing them with a query. 
     * This method is configured through the SimilarityConfig object.
     */
	public Collection<CBRCase> runProbabilityScoring(Collection<CBRCase> cases, CBRQuery query, int bnValues){
		HashMap<String,Double> kBNValues = getKHighestBNValues(query, bnValues);
		Collection<CBRCase> bnCases = new ArrayList<CBRCase>();
		//Put the relevant cases in the bnCases arraylist
		bnCases = getRelevantCases(cases, kBNValues);

		return bnCases;
	}
	
	/**
	 * This method sets the evidence in the BNnet and updates the belief
	 * @param query - The chosen values of food, acc and sauce
	 * @return HashMap with the grapes and their probabilities
	 */
	public HashMap getBNValues(CBRQuery query){
		//Find the attributes of the query
		String[] queryArray = query.getDescription().toString().split(";");
		String mealId = queryArray[0].substring(1);
		String foodType = queryArray[1].substring(1);
		String accType = queryArray[2].substring(2, queryArray[2].length()-1);
		String sauceType = queryArray[3].substring(1, queryArray[3].length()-1);
		
		
		//Set evidence based on the new cases' values
		if(foodType != null && !foodType.equals("null"))
			mealsNetwork.setEvidence(foodOntoClass, getOntoClass(foodType));
		if(!sauceType.isEmpty())
			mealsNetwork.setEvidence(sauceClassName, sauceType);
		if(accType != null && !accType.equals("null"))
			mealsNetwork.setEvidence(accOntoClass, getOntoClass(accType));
		mealsNetwork.updateBeliefs();
		
		System.out.println("");
		HashMap<String,Double> probabilities = mealsNetwork.getNodeProbForAllGrapes(wineGrapes, "True");
		System.out.println("\nValues from the query:");
		System.out.println("FoodType: " + foodType +" || SauceType: " + sauceType + " || AccType: " + accType);
		
		return probabilities;
	}
	
	/**
	 * This method finds all the values from the nodes
	 * @param query
	 * @return HashMap<String,Double>
	 */
	
	public HashMap<String, Double> getAllBNValues(CBRQuery query){
		//Find the attributes of the query
		String[] queryArray = query.getDescription().toString().split(";");
//		String mealId = queryArray[0].substring(1);
		String foodType = queryArray[1].substring(1);
		String accType = queryArray[2].substring(2, queryArray[2].length()-1);
		String sauceType = queryArray[3].substring(1, queryArray[3].length()-1);
		
		//Set evidence based on the new cases' values
		if(foodType != null && !foodType.equals("null"))
			mealsNetwork.setEvidence(foodOntoClass, getOntoClass(foodType));
		if(!sauceType.isEmpty())
			mealsNetwork.setEvidence(sauceClassName, sauceType);
		if(accType != null && !accType.equals("null"))
			mealsNetwork.setEvidence(accOntoClass, getOntoClass(accType));
		mealsNetwork.updateBeliefs();
		
		System.out.println("Returning all values from query: ");

		HashMap<String,Double> probabilities = new HashMap<String, Double>();
		for(int i = 0; i < 6; i++) {
			String[] nodeNameArray = getArray(i);
			String nodeName = nodeNameArray[0];
			System.out.println("\n nodeName sent is: " + nodeName);
			HashMap<String, Double> currentProbabilities = mealsNetwork.getAllNodeProbabilites(nodeName, nodeNameArray);
			probabilities.putAll(currentProbabilities);
		}
		
		System.out.println("\nValues from the query:");
		System.out.println("FoodType: " + foodType +" || SauceType: " + sauceType + " || AccType: " + accType);
		
		return probabilities;
	}
	
	/**
	 * This method calculates the log-odds ratio values.
	 * @param bnValues - a HashMap with the probabilities
	 * @return a HashMap with the log-odds ratio values
	 */
	public HashMap<String,Double> getLogOddsValues(HashMap<String,Double> bnValues){
		HashMap<String,Double> logValues = new HashMap<String,Double>();
		for(String key : bnValues.keySet()){
			double grapeProb = mealsNetwork.getNodeProb(key, "True", null, null, true);
			double log = Math.log(bnValues.get(key)/grapeProb);
			logValues.put(key, log);
		}
		return logValues;
	}
	
	/**
	 * This method gets the k highest grape-values from the BN. 
	 * @param query - The chosen values of food, acc and sauce
	 * @param k - the number of values to be retrieved
	 * @return a HashMap with the k highest grapevalues
	 */
	public HashMap<String,Double> getKHighestBNValues(CBRQuery query, int k){
		//Traverse HashMap and find the highest value
		HashMap<String,Double> bnValues = getBNValues(query);
		HashMap<String,Double> kValues = new HashMap<String,Double>(); //Saves the kHighest values
		
		//Get the log-values
		HashMap<String,Double> logValues = getLogOddsValues(bnValues);
		
		System.out.println("");
		String highestKey = null;
		double highestLog = -10.0;
		for(int i = 0; i < k; i++){
			for(String key : logValues.keySet()){
				if(logValues.get(key) > highestLog && !kValues.containsKey(key)){
					highestKey = key;
					highestLog = logValues.get(key);
				}
			}
			//Put the highest key into the list with the keys
			kValues.put(highestKey, bnValues.get(highestKey));
			
			//Prints the highest grapes to the console
			DecimalFormat df = new DecimalFormat("#.##", DecimalFormatSymbols.getInstance(Locale.ENGLISH));
			System.out.println((i+1)+". highest grape: "+highestKey + " with probability: "+df.format(bnValues.get(highestKey)*100)+" %  and log: "+ df.format(logValues.get(highestKey)));
			
			//Set highestkey and prob back to null and 0 before next iteration
			highestKey = null;
			highestLog = -10.0;
		}
		System.out.println("\n");
		return kValues;
	}
	
	
	/**
	 * This method finds the relevant cases of the given grapes
	 * @param cases - All cases in the case base
	 * @param grapeValues - The grapeValues that we are looking for in the cases
	 * @return a Collection of the relevant cases
	 */
	public Collection<CBRCase> getRelevantCases(Collection<CBRCase> cases, HashMap<String,Double> grapeValues){
		Collection<CBRCase> relCases = new ArrayList<CBRCase>();
		//M ha case solutions for  matche p druer!
		for(CBRCase _case: cases){
			MealSolution sol = (MealSolution)_case.getSolution();
			Set<GrapeDescription> grapes = sol.getGrapeType();
			
			for(GrapeDescription grape: grapes){
				if(grapeValues.containsKey(grape.toString())){
					if(!relCases.contains(_case)) {
						relCases.add(_case);
					}
				}
			}
		}
		return relCases;
	}

}