package no.ntnu.idi.bacchus.explanation;

import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Set;

import com.hp.hpl.jena.graph.query.QueryNode.JustBound;

import no.ntnu.idi.bacchus.MealRecommender;
import no.ntnu.idi.bacchus.bn.MealsNetwork;
import no.ntnu.idi.bacchus.bn.ProbabilityScoring;
import no.ntnu.idi.bacchus.domain.MealDescription;
import no.ntnu.idi.bacchus.domain.MealSolution;
import no.ntnu.idi.bacchus.gui.SimilarityDialog;

import jcolibri.cbrcore.*;
import jcolibri.method.retrieve.RetrievalResult;
import jcolibri.method.retrieve.NNretrieval.similarity.local.ontology.OntDeepBasic;

public class MealExplanation {

	String cbrExplanation;
	String bnExplanation;
	ArrayList<RetrievalResult> cases;
	
	Double foodTypeSim = 0.0;
	Double accTypeSim = 0.0;
	Double sauceTypeSim;
	public HashMap<String, Double> kHighestBNValues; 
	String CBRExplanation = "";
	String BNExplanation  = "" ;
	String CombinedExplanation = "";
	
	
	
//	private MealsNetwork mealsNetwork;
	
	public MealExplanation() {
	
	}
	
	
	public void setKHighestBNValues(HashMap<String, Double> hashMap) {
		this.kHighestBNValues = hashMap;
	}
	
	public String generateBNExplanation(CBRCase _case, CBRQuery query) {
		ProbabilityScoring probScore = MealRecommender.getProbScoring();
		HashMap<String,Double> probabilities = probScore.getAllBNValues(query);
		HashMap<String,Double> sortedValues = new HashMap<String, Double>();
		String[] probabilitiesArray = new String[probabilities.size()];
		
		System.out.println("Probabilities: ");
		printProbabilities(probabilities);
		
//		Sorts out the numberOfConditions# highest values from the intermediary nodes
		String highestKey = null;
		double highestValue = 0.0;
		int numberOfConditions = 6;
		for(int i = 0; i < numberOfConditions; i++){
			for(String key : probabilities.keySet()){
				if(probabilities.get(key) > highestValue && !sortedValues.containsKey(key)){
					highestKey = key;
					highestValue = probabilities.get(key);
				}
			}
			//Put the highest key into the list with the keys
			sortedValues.put(highestKey, probabilities.get(highestKey));
			//Set highestkey and prob back to null and 0 before next iteration
			highestKey = null;
			highestValue = 0.0;
		}
		
//		TODO: Sort out the highest probability for each nodeName and add them to list.
		
		System.out.println("\n Sorted Probabilities: ");
		printProbabilities(sortedValues);
		formatValues(sortedValues);
		System.out.println("\nFormatted values: \n");
		printProbabilities(sortedValues);
	
		probabilitiesArray = probabilitiesToStringArray(sortedValues);
		
		BNExplanation = BNExplanation + "\n" + "The meal-ingredients chosen is likely to result in the following characteristics: ";
		for(int j = probabilitiesArray.length / 2; j < probabilitiesArray.length; j++) 
			BNExplanation = BNExplanation + "\n" + probabilitiesArray[j] + "%";
		
		
		BNExplanation = BNExplanation + "\n" + "\n" + "Based on the meal's characteristics, the characteristics of the wine should meet the following requirements: ";
		for(int j = 0; j < probabilitiesArray.length / 2; j++) 
			BNExplanation = BNExplanation + "\n" + probabilitiesArray[j] + "%";
		
		
		BNExplanation = BNExplanation +"\n" + "\nThe grapes most likely to meet these are: ";
		
		kHighestBNValues = probScore.getKHighestBNValues(query, 3);
		formatValues(kHighestBNValues);
		String[] wineProbabilities = probabilitiesToStringArray(kHighestBNValues);
		for(int z = 0; z < wineProbabilities.length; z++) 
			BNExplanation = BNExplanation + "\n" + wineProbabilities[z] + "%";
		
		return BNExplanation;
	}
	
	void printProbabilities(HashMap<String,Double> hashMap) {
		System.out.println("Probabilities: ");
		Set<?> probabilityList = hashMap.entrySet();
		Iterator<?> it = probabilityList.iterator();
		while(it.hasNext()) {
			Object prob = it.next().toString();
			System.out.println(prob);
		}
	}
	
	String[] probabilitiesToStringArray(HashMap<String,Double> hashMap) {
		HashMap<String,String> stringMap = formatValues(hashMap);
		Set<?> probabilityList = stringMap.entrySet();
		String[] stringArray = new String[stringMap.size()];
		Iterator<?> it = probabilityList.iterator();
		while(it.hasNext()) {
			for(int i = 0; i < stringArray.length; i++) {
				Object prob = it.next().toString();
				stringArray[i] = prob.toString();
			}
		}
		return stringArray;
	}
	
	
	public String generateCombinedExplanation(CBRCase _case, CBRQuery query) {
		CBRExplanation = generateCBRExplanation(_case, query);
		BNExplanation = generateBNExplanation(_case, query);
		
		CombinedExplanation = CBRExplanation + "\n" +  BNExplanation; 
//		CombinedExplanation = CombinedExplanation + "\n" + "\nThe characteristics of the wine recommends the given grapes which match the given recipe's choice. ";
		return CombinedExplanation;
	}
	
	public String generateCBRExplanation(CBRCase _case, CBRQuery query) {
		
		MealDescription mealDesc = (MealDescription) _case.getDescription();
		MealSolution mealSol = (MealSolution) _case.getSolution();
		MealDescription queryDesc = (MealDescription) query.getDescription();
		
		
		no.ntnu.idi.bacchus.onto.OntDeepBasic sim = new no.ntnu.idi.bacchus.onto.OntDeepBasic();
		try {
		foodTypeSim = sim.compute(mealDesc.getFoodType(), queryDesc.getFoodType());
		accTypeSim = sim.compute(mealDesc.getAccessoriesType(), queryDesc.getAccessoriesType());
		sauceTypeSim = sim.compute(mealDesc.getSauceType(), queryDesc.getSauceType());
		} catch(Exception e) {
		}
/*
		if(mealDesc.getSauceType().equals(queryDesc.getSauceType()))
			sauceTypeSim = 1;
		else
			sauceTypeSim = 0;
*/	
		CBRExplanation = "The similarity between " + mealDesc.getFoodType() + " and " + queryDesc.getFoodType() +
			" is " + foodTypeSim + ". \nCombined with the chosen accessorie " + queryDesc.getAccessoriesType() +
			" which match with the proposed meal's " + mealDesc.getAccessoriesType() + " with similarity " + 
			accTypeSim + " \nThe sauce has similarity " + sauceTypeSim + "." + "\nThe mean of these values give the " +
			"similarity stated and is the most similar case in the case-base.";
		
		return CBRExplanation;
	}


	HashMap<String,String> formatValues(HashMap<String,Double> bnValues) {
		HashMap<String,String> stringMap = new HashMap<String,String>();
		for(String key: bnValues.keySet()) {
			double value = bnValues.get(key);
			value = value*100;
			DecimalFormat format = new DecimalFormat("##.##", DecimalFormatSymbols.getInstance(Locale.US));
			String formatValue = format.format(value);
//			DecimalFormat format2 = new DecimalFormat("##.###", DecimalFormatSymbols.getInstance(Locale.US));
//			System.out.println(key + "\n has new prob: " + format.format(bnValues.get(key)*100) + "\n and old prob " + 
//				format.format(grapeProb*100) + "\n and odds ratio " + (bnValues.get(key)/grapeProb) + "\n and log: " +
//				format2.format(log)+"\n");
			stringMap.put(key, formatValue);
	
		}
		return stringMap;
	}


}
