package no.ntnu.idi.bacchus.gui;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collection;

import javax.swing.*;
import javax.swing.border.Border;
import javax.swing.text.StyledEditorKit.FontFamilyAction;

import no.ntnu.idi.bacchus.MealRecommender;
import no.ntnu.idi.bacchus.domain.MealDescription;
import no.ntnu.idi.bacchus.domain.MealSolution;
import no.ntnu.idi.bacchus.explanation.MealExplanation;

import jcolibri.cbrcore.CBRCase;
import jcolibri.cbrcore.CBRQuery;
import jcolibri.method.retrieve.RetrievalResult;
import jcolibri.method.retrieve.NNretrieval.NNConfig;
import jcolibri.method.retrieve.NNretrieval.similarity.local.ontology.OntDeepBasic;
import jcolibri.util.FileIO;

// Explanation Dialog class

public class ExplanationDialog extends JDialog {

	private static final long serialVersionUID = 1L;
	
	public enum ReasoningTypes { CBR, BN, Combination }
	
	ReasoningTypes reasoningTypes;
	CBRQuery query;
	CBRCase _case;
	
	MealDescription mealDesc;
	MealSolution mealSol;
	
	JLabel image;
	JLabel foodType;
	JLabel accessoriesType;
	JLabel sauceType;
	JLabel grapeType;
	JLabel wineTitle;
	JLabel recipeTitle;
	JLabel wineOrigin;
	JLabel similarityLabel;
	JTextPane explanationBox;
	
	private String explanationText;
	String similarity;
	
	
	public ExplanationDialog(JFrame main)
	{
		super(main,true);
		configureFrame();
	}
	
	/**
	 * Configures the look and feel of the frame.
	 */
	
	private void configureFrame()
	{
		try
		{
		    UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e1)
		{
		}
		
		this.setTitle("Explanation");

		
		image = new JLabel();
		image.setIcon(new ImageIcon(FileIO.findFile("no/ntnu/idi/bacchus/gui/lateral.png")));
		this.getContentPane().setLayout(new BorderLayout());
		this.getContentPane().add(image, BorderLayout.WEST);
	
		/**********************************************************/
		
		JPanel descriptionPanel = new JPanel();
		//panel.setLayout(new GridLayout(8,2));
		descriptionPanel.setLayout(new SpringLayout());
		JLabel label;

//		***** Content Layout *****
		
		descriptionPanel.add(label = new JLabel("Description"));
		label.setFont(label.getFont().deriveFont(Font.BOLD));
		descriptionPanel.add(label = new JLabel());

		
		descriptionPanel.add(new JLabel("FoodType"));
		descriptionPanel.add(foodType = new JLabel(""));
		
		descriptionPanel.add(new JLabel("Accessories"));
		descriptionPanel.add(this.accessoriesType = new JLabel());
		
		descriptionPanel.add(new JLabel("Sauce"));
		descriptionPanel.add(this.sauceType = new JLabel());
		
		
		descriptionPanel.add(label = new JLabel("Solution"));
		label.setFont(label.getFont().deriveFont(Font.BOLD));
		descriptionPanel.add(label = new JLabel());

		descriptionPanel.add(new JLabel("RecipeTitle"));
		descriptionPanel.add(this.recipeTitle = new JLabel());
		
		descriptionPanel.add(new JLabel("WineTitle"));
		descriptionPanel.add(this.wineTitle = new JLabel());
		
		descriptionPanel.add(new JLabel("WineOrigin"));
		descriptionPanel.add(this.wineOrigin = new JLabel());
		
		descriptionPanel.add(new JLabel("Grapes"));
		descriptionPanel.add(this.grapeType = new JLabel());
		
		descriptionPanel.add(new JLabel("Similarity"));
		descriptionPanel.add(this.similarityLabel = new JLabel());

//		JPanel explanationPanel = new JPanel();
//		explanationPanel.setLayout(new SpringLayout());

		descriptionPanel.add(new JLabel("Explanation:"));
		
		
		explanationBox = new JTextPane();
		explanationBox.setPreferredSize(new Dimension(600,300));
		descriptionPanel.add(this.explanationBox);
/*		
		Utils.makeCompactGrid(explanationPanel, 
						2, 1, 
						6, 6, 
						30, 10);
*/	
		
		
//		Lay out the panel.
		Utils.makeCompactGrid(descriptionPanel,
		                11, 2, //rows, cols
		                6, 6,        //initX, initY
		                30, 10);       //xPad, yPad
		
		
		
//		****** Content end *****

		
		JPanel auxPanel = new JPanel();
		auxPanel.setLayout(new BorderLayout());
		auxPanel.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));

//		auxPanel.add(explanationPanel, BorderLayout.NORTH);
		auxPanel.add(descriptionPanel,BorderLayout.NORTH);
		
		
		JPanel buttons = new JPanel();
		buttons.setLayout(new BorderLayout());
		
		JButton back = new JButton("<< Back to Result");
		back.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				back();
			}
		});
		buttons.add(back,BorderLayout.SOUTH);
		
/*		
		JButton exit = new JButton("Exit");
		exit.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				try {
					MealRecommender.getInstance().postCycle();
				} catch (Exception ex) {
					org.apache.commons.logging.LogFactory.getLog(MealRecommender.class).error(ex);
				}
				System.exit(-1);
			}
		});
		buttons.add(exit,BorderLayout.WEST);
*/
		
		JButton bn = new JButton("BN");
		bn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				generateExplanation(_case, query, ReasoningTypes.BN);
				showExplanation();
			}
		});
		buttons.add(bn,BorderLayout.EAST);
		
		JButton cbr = new JButton("CBR");
		cbr.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				generateExplanation(_case, query, ReasoningTypes.CBR);
				showExplanation();
			}
		});
		buttons.add(cbr,BorderLayout.WEST);
		
		JButton combo = new JButton("Combination");
		combo.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				generateExplanation(_case, query, ReasoningTypes.Combination);
				showExplanation();
			}
		});
		buttons.add(combo, BorderLayout.CENTER);
		
		auxPanel.add(buttons, BorderLayout.SOUTH);
		this.getContentPane().add(auxPanel, BorderLayout.CENTER);
		
		/**********************************************************/
		
		this.pack();
		this.setSize(1000, this.getHeight());
		this.setResizable(false);
		Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
		setBounds((screenSize.width - this.getWidth()) / 2,
			(screenSize.height - this.getHeight()) / 2, 
			getWidth(),
			getHeight());
		
	}
	
	public void generateExplanation(CBRCase _case, CBRQuery query, ReasoningTypes reasoningTypes) {
		MealExplanation mealExplanation = new MealExplanation();
		
		switch(reasoningTypes) {
			case CBR: setExplanation(mealExplanation.generateCBRExplanation(_case, query)); break;
			case BN: setExplanation(mealExplanation.generateBNExplanation(_case, query)); break;
			case Combination: setExplanation(mealExplanation.generateCombinedExplanation(_case, query)); break;
		}
	}
	
	/**
	 * Takes the @param cases and @param currentCase as input from ResultDialog.java 
	 * to show the correct case explanation.
	 * @param cases
	 * @param currentCase
	 */
	public void setCurrentCaseInfo(ArrayList<RetrievalResult> cases, int currentCase, CBRQuery query) {
		RetrievalResult retrievalResult = cases.get(currentCase);
		
		this.query = query;
		this.similarity = Double.toString(retrievalResult.getEval());
		this._case = retrievalResult.get_case();
		this.mealDesc = (MealDescription) _case.getDescription();
		this.mealSol = (MealSolution) _case.getSolution();
//		MealDescription queryDesc = (MealDescription) query.getDescription();
		
	}		
	
	public void showExplanation() {
		this.foodType.setText(mealDesc.getFoodType().toString());
		this.accessoriesType.setText(mealDesc.getAccessoriesType().toString());
		this.sauceType.setText(mealDesc.getSauceType().toString());
		
		this.grapeType.setText(mealSol.getGrapeType().toString());
		this.wineOrigin.setText(mealSol.getWineOrigin().toString());
		this.wineTitle.setText(mealSol.getWineTitle().toString());
		
		this.similarityLabel.setText(similarity);
		this.explanationBox.setText(explanationText);
		
		
//		Trace reasoning steps
	}
	
	/**
	 * Close the explanation and go back to ResultDialog.java
	 */
	void back() {
		this.setVisible(false);
	}
	
	public void setExplanation(String text) {
		this.explanationText = text;
	}
	
	
	public static void main(String[] args) {
		ExplanationDialog explanationDialog = new ExplanationDialog(null);
		explanationDialog.setVisible(true);
		System.out.println("Bye");
	}
}
