/**
 * Travel Recommender example for the jCOLIBRI2 framework. 
 * @author Juan A. Recio-Garca.
 * GAIA - Group for Artificial Intelligence Applications
 * http://gaia.fdi.ucm.es
 * 25/07/2006
 */
package no.ntnu.idi.bacchus.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashSet;
import java.util.Set;

import javax.swing.*;

import no.ntnu.idi.bacchus.MealRecommender;
import no.ntnu.idi.bacchus.domain.MealDescription;

import jcolibri.cbrcore.CBRQuery;
import jcolibri.datatypes.Instance;
import jcolibri.util.FileIO;

/**
 * Query dialgo
 * @author Juan A. Recio-Garcia
 * @version 1.0
 */
public class QueryDialog extends JDialog {

	private static final long serialVersionUID = 1L;
	JLabel image;
	
	//SpinnerNumberModel numberOfPersons;
	FoodSelector foodSelector;
	AccessoriesSelector accessoriesSelector;
	JComboBox sauceType;
	//JComboBox wineOrigin;
	//JComboBox grapeType;
	
	public QueryDialog(JFrame parent)
	{
		super(parent,true);
		configureFrame();
	}
	
	private void configureFrame(){
		try{
		    UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e1){}
		this.setTitle("BACCHUS - Configure Query");
		
		image = new JLabel();
		image.setIcon(new ImageIcon(FileIO.findFile("no/ntnu/idi/bacchus/gui/step1.png")));
		this.getContentPane().setLayout(new BorderLayout());
		this.getContentPane().add(image, BorderLayout.WEST);
		
		/**********************************************************/
		JPanel panel = new JPanel();
		//panel.setLayout(new GridLayout(8,2));
		panel.setLayout(new SpringLayout());
		JLabel label;
		panel.add(label = new JLabel("Attribute"));
		label.setFont(label.getFont().deriveFont(Font.BOLD));
		panel.add(label = new JLabel("Value"));
		label.setFont(label.getFont().deriveFont(Font.BOLD));

		panel.add(new JLabel("Food"));
		panel.add(foodSelector = new FoodSelector(this));
		
		panel.add(new JLabel("Accessories")); //Skal kunne velge flere!
		panel.add(accessoriesSelector = new AccessoriesSelector(this));
		
		panel.add(new JLabel("Sauce"));
		String[] sauceTypes = {"","BearnaiseSauce", "CreamSauce", "GameSauce", "GravySauce", "MushroomSauce", "ProvenceSauce", "RedWineSauce", "TomatoSauce", "NoSauce"};
		panel.add(sauceType = new JComboBox(sauceTypes));
		
		/**panel.add(new JLabel("WineOrigin"));
		String[] transportations = {"Argentina", "Australia", "California", "Chile", "France", "Italy", "NewZealand", "Portugal", "Spain", "SouthAfrica"};
		panel.add(wineOrigin = new JComboBox(transportations));
		
		panel.add(new JLabel("GrapeType"));
		String[] seasons = {"Barbera", "CabernetFranc", "CabernetSauvignon", "Carignan", "Corvina", "Grenache", "Malbec", "Merlot", "Montepulciano", "PinotNoir", "Rondinella", "Sangiovese", "Syrah", "Tempranillo", "TintaMiuda", "Zinfandel"};
		panel.add(grapeType = new JComboBox(seasons));**/
		
//		Lay out the panel.
		Utils.makeCompactGrid(panel,
		                4, 2, //rows, cols
		                6, 6,        //initX, initY
		                10, 10);       //xPad, yPad
		
		JPanel panelAux = new JPanel();
		panelAux.setLayout(new BorderLayout());
		panelAux.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));

		panelAux.add(panel,BorderLayout.NORTH);
		JPanel buttons = new JPanel();
		buttons.setLayout(new BorderLayout());
		
		JButton ok = new JButton("Set Query >>");
		ok.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				setQuery();
			}
		});
		buttons.add(ok,BorderLayout.CENTER);
		JButton exit = new JButton("Exit");
		exit.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				try {
					MealRecommender.getInstance().postCycle();
				} catch (Exception ex) {
					org.apache.commons.logging.LogFactory.getLog(MealRecommender.class).error(ex);
				}
				System.exit(-1);
			}
		});
		buttons.add(exit,BorderLayout.WEST);
		
		panelAux.add(buttons, BorderLayout.SOUTH);
		this.getContentPane().add(panelAux, BorderLayout.CENTER);
		
		/**********************************************************/
		
		this.pack();
		this.setSize(600, this.getHeight());
		this.setResizable(false);
		Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
		setBounds((screenSize.width - this.getWidth()) / 2,
			(screenSize.height - this.getHeight()) / 2, 
			getWidth(),
			getHeight());
	}
	
	
	void setQuery(){
		this.setVisible(false);
	}
	
	public CBRQuery getQuery(){
		MealDescription desc = new MealDescription();
		Set<Instance> accessoriesType = new HashSet<Instance>();
		accessoriesType.add(this.accessoriesSelector.getSelectedInstance());
		
		desc.setFoodType(this.foodSelector.getSelectedInstance());
		desc.setAccessoriesType(accessoriesType);
		desc.setSauceType((String)this.sauceType.getSelectedItem());
		
		CBRQuery query = new CBRQuery();
		query.setDescription(desc);
		
		return query;
	}
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		QueryDialog qf = new QueryDialog(null);
		qf.setVisible(true);
		System.out.println("Bye");
	}

	

}
