/**
 * Travel Recommender example for the jCOLIBRI2 framework. 
 * @author Juan A. Recio-Garca.
 * GAIA - Group for Artificial Intelligence Applications
 * http://gaia.fdi.ucm.es
 * 25/07/2006
 */
package no.ntnu.idi.bacchus.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Locale;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SpringLayout;
import javax.swing.UIManager;

import no.ntnu.idi.bacchus.MealRecommender;
import no.ntnu.idi.bacchus.domain.MealDescription;
import no.ntnu.idi.bacchus.domain.MealSolution;

import jcolibri.cbrcore.CBRCase;
import jcolibri.cbrcore.CBRQuery;
import jcolibri.method.retrieve.RetrievalResult;
import jcolibri.util.FileIO;

/**
 * Result dialog
 * @author Juan A. Recio-Garcia
 * @version 1.0
 */
public class ResultDialog extends JDialog {

	private static final long serialVersionUID = 1L;

	JLabel image;

	JLabel recipeTitle;
	JLabel foodType;
	JLabel accessoriesType;
	JLabel grapeType;
	JLabel sauceType;
	JLabel wineOrigin;
	JLabel wineTitle;
	JLabel caseId;
	
	CBRQuery query;
	ArrayList<RetrievalResult> cases;
	int currentCase;
	
	
	public ResultDialog(JFrame main)
	{
		super(main,true);
		configureFrame();
	}
	
	private void configureFrame()
	{
		try
		{
		    UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e1)
		{
		}
		
		this.setTitle("BACCHUS - Retrived cases");

		
		image = new JLabel();
		image.setIcon(new ImageIcon(FileIO.findFile("no/ntnu/idi/bacchus/gui/step3.png")));
		this.getContentPane().setLayout(new BorderLayout());
		this.getContentPane().add(image, BorderLayout.WEST);
		
		
		/**********************************************************/
		JPanel panel = new JPanel();
		//panel.setLayout(new GridLayout(8,2));
		panel.setLayout(new SpringLayout());
		
		JLabel label;

		panel.add(label = new JLabel("Description"));
		label.setFont(label.getFont().deriveFont(Font.BOLD));
		panel.add(label = new JLabel());

		
		panel.add(new JLabel("Food"));
		panel.add(foodType = new JLabel("adsfadfadsf"));
		
		panel.add(new JLabel("Accessories"));
		panel.add(this.accessoriesType = new JLabel());
		
		panel.add(new JLabel("Sauce"));
		panel.add(this.sauceType = new JLabel());
		
		
		panel.add(label = new JLabel("Solution"));
		label.setFont(label.getFont().deriveFont(Font.BOLD));
		panel.add(label = new JLabel());

		panel.add(new JLabel("Recipe title"));
		panel.add(this.recipeTitle = new JLabel());
		
		panel.add(new JLabel("Wine title"));
		panel.add(this.wineTitle = new JLabel());
		
		panel.add(new JLabel("Wine origin"));
		panel.add(this.wineOrigin = new JLabel());
		
		panel.add(new JLabel("Grapes"));
		panel.add(this.grapeType = new JLabel());
		
//		Lay out the panel.
		Utils.makeCompactGrid(panel,
		                9, 2, //rows, cols
		                6, 6,        //initX, initY
		                30, 10);       //xPad, yPad
		
		JPanel casesPanel = new JPanel();
		casesPanel.setLayout(new BorderLayout());
		casesPanel.add(panel, BorderLayout.CENTER);
		
		JPanel casesIterPanel = new JPanel();
		casesIterPanel.setLayout(new FlowLayout());
		JButton prev = new JButton("<<");
		casesIterPanel.add(prev);
		casesIterPanel.add(caseId = new JLabel("Case id"));
		JButton follow = new JButton(">>");
		casesIterPanel.add(follow);
		
		prev.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				currentCase = (currentCase+cases.size()-1) % cases.size();
				showCase();
			}
		});
		
		follow.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				currentCase = (currentCase+1) % cases.size();
				showCase();
			}
		});
		
		casesPanel.add(casesIterPanel, BorderLayout.NORTH);
		
		JPanel panelAux = new JPanel();
		panelAux.setLayout(new BorderLayout());
		panelAux.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));

		panelAux.add(casesPanel,BorderLayout.NORTH);
		
		JPanel buttons = new JPanel();
		buttons.setLayout(new BorderLayout());
		
		JButton ok = new JButton("Next >>");
		ok.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				next();
			}
		});
		buttons.add(ok,BorderLayout.EAST);
		
		JButton exit = new JButton("Exit");
		exit.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				try {
					MealRecommender.getInstance().postCycle();
				} catch (Exception ex) {
					org.apache.commons.logging.LogFactory.getLog(MealRecommender.class).error(ex);
				}
				System.exit(-1);
			}
		});
		buttons.add(exit,BorderLayout.WEST);
		
//		***** Explanations *****
		JButton explanation = new JButton("Explanation");
		explanation.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				giveExplanation();
			}
		});
		buttons.add(explanation,BorderLayout.CENTER);
//		***** End *****

		panelAux.add(buttons, BorderLayout.SOUTH);
		this.getContentPane().add(panelAux, BorderLayout.CENTER);	
		/**********************************************************/
		
		
		this.pack();
		this.setSize(600, this.getHeight());
		this.setResizable(false);
		Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
		setBounds((screenSize.width - this.getWidth()) / 2,
			(screenSize.height - this.getHeight()) / 2, 
			getWidth(),
			getHeight());
	}
	
	void next()
	{
		this.setVisible(false);
	}
	
	
	public void showCases(Collection<RetrievalResult> eval, Collection<CBRCase> selected, CBRQuery query)
	{
		cases = new ArrayList<RetrievalResult>();
		for(RetrievalResult rr: eval)
			if(selected.contains(rr.get_case()))
				cases.add(rr);
		this.query = query;
		currentCase = 0;
		showCase();
	}

	void giveExplanation() {
//		this.setVisible(false);
		ExplanationDialog explanationDialog = new ExplanationDialog(null);
		explanationDialog.setCurrentCaseInfo(cases, currentCase, query);
		explanationDialog.setVisible(true);
	}
	
	

	void showCase()
	{
		RetrievalResult rr = cases.get(currentCase);
		double sim = rr.getEval();
		
		CBRCase _case = rr.get_case();
		this.caseId.setText(_case.getID().toString()+" -> "+sim+" ("+(currentCase+1)+"/"+cases.size()+")");
		
		MealDescription desc = (MealDescription) _case.getDescription();
		
		this.foodType.setText(desc.getFoodType().toString());
		this.accessoriesType.setText(desc.getAccessoriesType().toString());
		this.sauceType.setText(desc.getSauceType());
		
		
		MealSolution sol = (MealSolution) _case.getSolution();
		this.wineOrigin.setText(sol.getWineOrigin());
		this.grapeType.setText(sol.printGrapes());
		this.wineTitle.setText(sol.getWineTitle());
		this.recipeTitle.setText(sol.getRecipeTitle());
		
	}
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		ResultDialog qf = new ResultDialog(null);
		qf.setVisible(true);
		System.out.println("Bye");
	}

	

}
