/**
 * Travel Recommender example for the jCOLIBRI2 framework. 
 * @author Juan A. Recio-Garca.
 * GAIA - Group for Artificial Intelligence Applications
 * http://gaia.fdi.ucm.es
 * 25/07/2006
 */
package no.ntnu.idi.bacchus.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import javax.swing.AbstractButton;
import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.SpinnerNumberModel;
import javax.swing.SpringLayout;
import javax.swing.UIManager;

import no.ntnu.idi.bacchus.MealRecommender;
import no.ntnu.idi.bacchus.domain.MealDescription;
import no.ntnu.idi.bacchus.domain.MealSolution;

import jcolibri.cbrcore.CBRCase;
import jcolibri.datatypes.Instance;
import jcolibri.util.FileIO;

/**
 * Revision Dialog
 * @author Juan A. Recio-Garcia
 * @version 1.0
 */
public class RevisionDialog extends JDialog {

	private static final long serialVersionUID = 1L;

	JLabel image;
	
	JComboBox sauceType;
	FoodSelector foodSelector;
	AccessoriesSelector accSelector;
	JComboBox grapeType;
	JComboBox wineOrigin;
	JLabel caseId;
	JTextField recipeTitle;
	JTextField wineTitle;
	
	/**
	 * FoodSelector foodSelector;
	AccessoriesSelector accessoriesSelector;
	JComboBox sauceType;
	JComboBox wineOrigin;
	JComboBox grapeType;
	
	JLabel recipeTitle;
	JLabel foodType;
	JLabel accessoriesType;
	JLabel sauceType;
	JLabel wineOrigin;
	JLabel wineTitle;
	JLabel caseId;
	 */
	
	ArrayList<CBRCase> cases;
	int currentCase;
	
	public RevisionDialog(JFrame main)
	{
		super(main, true);
		configureFrame();
	}
	
	private void configureFrame()
	{
		try
		{
		    UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e1)
		{
		}
		
		this.setTitle("Revise Cases");

		
		image = new JLabel();
		image.setIcon(new ImageIcon(FileIO.findFile("no/ntnu/idi/bacchus/gui/step5.png")));
		this.getContentPane().setLayout(new BorderLayout());
		this.getContentPane().add(image, BorderLayout.WEST);
		
		
		/**********************************************************/
		JPanel panel = new JPanel();
		//panel.setLayout(new GridLayout(8,2));
		panel.setLayout(new SpringLayout());
		
		JLabel label;

		panel.add(label = new JLabel("Description"));
		label.setFont(label.getFont().deriveFont(Font.BOLD));
		panel.add(label = new JLabel());

		panel.add(new JLabel("FoodType"));
		panel.add(foodSelector = new FoodSelector(this));
		
		panel.add(new JLabel("AccessoriesType")); //Skal kunne velge flere!
		panel.add(accSelector = new AccessoriesSelector(this));
		
		panel.add(new JLabel("SauceType"));
		String[] sauceTypes = {"BernaiseSauce", "CreamSauce", "GameSauce", "GravySauce", "MushroomSauce", "ProvanceSauce", "RedWineSauce", "TomatoSauce"};
		panel.add(sauceType = new JComboBox(sauceTypes));
		
		panel.add(new JLabel("WineOrigin"));
		String[] transportations = {"Argentina", "Australia", "California", "Chile", "France", "Italia", "NewZealand", "Portugal", "Spain", "SouthAfrica"};
		panel.add(wineOrigin = new JComboBox(transportations));
		
		panel.add(new JLabel("GrapeType"));
		String[] seasons = {"Barbera", "CabernetFranc", "CabernetSauvignon", "Carignan", "Corvina", "Grenache", "Malbec", "Merlot", "Montepulciano", "PinotNoir", "Rondinella", "Sangiovese", "Syrah", "Tempranillo", "TintaMiuda", "Zinfandel"};
		panel.add(grapeType = new JComboBox(seasons));
		
		
		panel.add(label = new JLabel("Solution"));
		label.setFont(label.getFont().deriveFont(Font.BOLD));
		panel.add(label = new JLabel());
		
		panel.add(new JLabel("RecipeTitle"));
		panel.add(this.recipeTitle = new JTextField());
		
		panel.add(new JLabel("WineTitle"));
		panel.add(wineTitle = new JTextField());
		
//		Lay out the panel.
		Utils.makeCompactGrid(panel,
		                9, 2, //rows, cols
		                6, 6,        //initX, initY
		                30, 10);       //xPad, yPad
		
		JPanel casesPanel = new JPanel();
		casesPanel.setLayout(new BorderLayout());
		casesPanel.add(panel, BorderLayout.CENTER);
		
		JPanel casesIterPanel = new JPanel();
		casesIterPanel.setLayout(new FlowLayout());
		JButton prev = new JButton("<<");
		casesIterPanel.add(prev);
		casesIterPanel.add(caseId = new JLabel("Case id"));
		JButton follow = new JButton(">>");
		casesIterPanel.add(follow);
		
		prev.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				saveCase();
				currentCase = (currentCase+cases.size()-1) % cases.size();
				showCase();
			}
		});
		
		follow.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				saveCase();
				currentCase = (currentCase+1) % cases.size();
				showCase();
			}
		});
		
		casesPanel.add(casesIterPanel, BorderLayout.NORTH);
		
		
		JPanel panelAux = new JPanel();
		panelAux.setLayout(new BorderLayout());
		panelAux.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));

		panelAux.add(casesPanel,BorderLayout.NORTH);
		
		JPanel buttons = new JPanel();
		buttons.setLayout(new BorderLayout());
		
		JButton ok = new JButton("Set Revisions >>");
		ok.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				saveCase();
				next();
			}
		});
		buttons.add(ok,BorderLayout.CENTER);
		JButton exit = new JButton("Exit");
		exit.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				try {
					MealRecommender.getInstance().postCycle();
				} catch (Exception ex) {
					org.apache.commons.logging.LogFactory.getLog(MealRecommender.class).error(ex);
				}
				System.exit(-1);
			}
		});
		buttons.add(exit,BorderLayout.WEST);
		
		panelAux.add(buttons, BorderLayout.SOUTH);
		this.getContentPane().add(panelAux, BorderLayout.CENTER);
		
		/**********************************************************/
		
		
		this.pack();
		this.setSize(600, this.getHeight());
		this.setResizable(false);
		Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
		setBounds((screenSize.width - this.getWidth()) / 2,
			(screenSize.height - this.getHeight()) / 2, 
			getWidth(),
			getHeight());
	}
	
	void next()
	{
		this.setVisible(false);
	}
	
	
	public void showCases(Collection<CBRCase> cases)
	{
		this.cases = new ArrayList<CBRCase>(cases);
		currentCase = 0;
		showCase();
	}
	
	void showCase()
	{
		CBRCase _case = cases.get(currentCase);
		this.caseId.setText(_case.getID().toString()+" ("+(currentCase+1)+"/"+cases.size()+")");
		
		MealDescription desc = (MealDescription) _case.getDescription();
		
		this.foodSelector.setSelectedInstance(desc.getFoodType());
		//this.accSelector.setSelectedInstance(desc.getAccessoriesType());
		this.sauceType.setSelectedItem(desc.getSauceType().toString());
		
		MealSolution sol = (MealSolution) _case.getSolution();
		this.grapeType.setSelectedItem(sol.getGrapeType());
		this.wineOrigin.setSelectedItem(sol.getWineOrigin());
		this.wineTitle.setText(sol.getWineTitle());
		this.recipeTitle.setText(sol.getRecipeTitle());
		
	}
	
	void saveCase()
	{
		CBRCase _case = cases.get(currentCase);
		this.caseId.setText(_case.getID().toString()+" ("+(currentCase+1)+"/"+cases.size()+")");
		
		MealDescription desc = (MealDescription) _case.getDescription();
		
		desc.setSauceType((String)this.sauceType.getSelectedItem());
		desc.setFoodType(this.foodSelector.getSelectedInstance());
		//desc.setAccessoriesType(this.accSelector.getSelectedInstance());
		
		MealSolution sol = (MealSolution) _case.getSolution();
		sol.setWineOrigin((String)this.wineOrigin.getSelectedItem());
		//sol.setGrapeType(this.grapeType.getSelectedInstance());
		sol.setRecipeTitle(this.recipeTitle.getText());
	}
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		RevisionDialog qf = new RevisionDialog(null);
		qf.setVisible(true);
		System.out.println("Bye");
	}

	

}
