/**
 * OntDeepBasic.java
 * jCOLIBRI2 framework. 
 * @author Juan A. Recio-Garca.
 * GAIA - Group for Artificial Intelligence Applications
 * http://gaia.fdi.ucm.es
 * 13/06/2007
 */
package no.ntnu.idi.bacchus.onto;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Set;

import no.ntnu.idi.bacchus.domain.AccessoriesDescription;

import jcolibri.datatypes.Instance;
import jcolibri.exception.NoApplicableSimilarityFunctionException;
import jcolibri.exception.OntologyAccessException;
import jcolibri.method.retrieve.NNretrieval.similarity.LocalSimilarityFunction;
import jcolibri.util.FileIO;
import es.ucm.fdi.gaia.ontobridge.OntoBridge;
import es.ucm.fdi.gaia.ontobridge.OntologyDocument;

/**
 * This function computes the fdeep_basic similarity. See package documentation
 * for details.
 * 
 * @author Juan A. Recio-Garcia
 * @version 1.0
 * @see jcolibri.method.retrieve.NNretrieval.similarity.local
 */
public class OntDeepBasic implements LocalSimilarityFunction{
	

	public Object getQueryObject(Object queryObject){
		Object nextObject = new Object();
		if(queryObject instanceof Set<?>){
			Iterator iter = ((Set<?>) queryObject).iterator();
			nextObject = new Object();
			while(iter.hasNext())
				nextObject = iter.next();
			return nextObject;
		}
		else
			return queryObject;	
	}
	
	public Object getCaseObject(Object caseObject){
		if(caseObject instanceof AccessoriesDescription){
    		String accName = ((AccessoriesDescription) caseObject).getAccessoriesName();
    		Instance newInstance;
			try {
				newInstance = new Instance(accName);
				return (Object)newInstance;
			} catch (OntologyAccessException e) {
				e.printStackTrace();
			}
    	}
		return caseObject;
	}
	
	public double computeSeveral(Object caseObject, Object queryObject) throws NoApplicableSimilarityFunctionException{
    	//System.out.println("CaseObject: "+ caseObject + " & queryObject: " + queryObject);
    	queryObject = getQueryObject(queryObject);
    	int numberOfCaseObjects = 0;
    	double sumProb = 0.0;
    	boolean isExact = false;
    	if(caseObject instanceof Set<?>){
    		Iterator iter = ((Set<?>) caseObject).iterator();
    		while(iter.hasNext()){
    			Object nextObject = iter.next();
    			numberOfCaseObjects++;
    			double temp = compute(getCaseObject(nextObject), getQueryObject(queryObject)); 
    			if(temp == 1.0){
    				isExact = true;
    				//System.out.println("Object " + nextObject +" is an exact copy of " + queryObject);
    			}
    			sumProb += temp;
    		}
    	}
    	double probability;
    	if(isExact)
    		probability = 1.0;
    	else
    		probability = sumProb/numberOfCaseObjects;
		return probability;
	}

    	/*
         * (non-Javadoc)
         * 
         * @see jcolibri.method.retrieve.NNretrieval.similarity.LocalSimilarityFunction#compute(java.lang.Object,
         *      java.lang.Object, jcolibri.cbrcore.CaseComponent,
         *      jcolibri.cbrcore.CaseComponent, jcolibri.cbrcore.CBRCase,
         *      jcolibri.cbrcore.CBRQuery, java.lang.String)
         */
    public double compute(Object caseObject, Object queryObject) throws NoApplicableSimilarityFunctionException
    {
	if ((caseObject == null) || (queryObject == null))
	    return 0;
	if(caseObject instanceof Set<?> || queryObject instanceof Set<?>)
		return computeSeveral(caseObject, queryObject);
	if (!(caseObject instanceof Instance))
	    throw new jcolibri.exception.NoApplicableSimilarityFunctionException(this.getClass(), caseObject.getClass());
	if (!(queryObject instanceof Instance))
	    throw new jcolibri.exception.NoApplicableSimilarityFunctionException(this.getClass(), queryObject
		    .getClass());

	Instance i1 = (Instance) caseObject;
	Instance i2 = (Instance) queryObject;

	
	if (i1.equals(i2)) {
		System.out.println("Similarity for query object: " + i2 + " compared to: " + i1 + " is: " + 1);
	    return 1;
	}

	OntoBridge ob = jcolibri.util.OntoBridgeSingleton.getOntoBridge();

	double up = ob.maxProfLCS(i1.toString(), i2.toString());
	double down = ob.getMaxProf();

	double similarity = up / down;
	System.out.println("Similarity for query object: " + i2 + " compared to: " + i1 + " is: " + similarity);
	
	return up / down;
    }

  
    
    /** Applicable to Instance */
    public boolean isApplicable(Object o1, Object o2)
    {
	if ((o1 == null) && (o2 == null))
	    return true;
	else if (o1 == null)
	    return o2 instanceof Instance;
	else if (o2 == null)
	    return o1 instanceof Instance;
	else
	    return (o1 instanceof Instance) && (o2 instanceof Instance);
    }

    /**
         * Testing method using test5 ontology
         */
    public static void main(String[] args)
    {
	try
	{
	    // Obtain a reference to OntoBridge
	    OntoBridge ob = jcolibri.util.OntoBridgeSingleton.getOntoBridge();
	    // Configure it to work with the Pellet reasoner
	    ob.initWithPelletReasoner();
	    // Setup the main ontology
	    OntologyDocument mainOnto = new OntologyDocument("http://gaia.fdi.ucm.es/ontologies/vacation.owl", FileIO
		    .findFile("jcolibri/test/test5/vacation.owl").toExternalForm());
	    // There are not subontologies
	    ArrayList<OntologyDocument> subOntologies = new ArrayList<OntologyDocument>();
	    // Load the ontology
	    ob.loadOntology(mainOnto, subOntologies, false);

	    OntDeepBasic sim = new OntDeepBasic();

	    System.out.println("deep_basic(CAR,TRAIN)=" + sim.compute(new Instance("CAR"), new Instance("TRAIN")));
	    System.out.println("deep_basic(CAR,IBIZA)=" + sim.compute(new Instance("CAR"), new Instance("IBIZA")));
	    System.out.println("deep_basic(CAR,I101)=" + sim.compute(new Instance("CAR"), new Instance("I101")));

	} catch (Exception e)
	{
	    org.apache.commons.logging.LogFactory.getLog(OntDeepBasic.class).error(e);
	}
    }
}
